#include "GenerateEnv.h"
#include <stdio.h>

char ** GenerateEnvironmentStrings(char *pszFileName)
{
	int iproc = 0;
	int nShmLow, nShmHigh;
	char **ppEnv = NULL;
	int nProc = 0;
	char pszTemp[1024];

	// Parse the config file into g_pHosts
	if (!ParseConfigFile(pszFileName))
		return NULL;

	if (g_bStaticEnvironment)
	{
		ppEnv = new char*[g_nNproc+1];
		for (int i=0; i<g_nNproc; i++)
		{
			// Create the string
			sprintf(pszTemp, "MPICH_NPROC=%d|MPICH_IPROC=%d", g_nNproc, i);
			
			// Add any user variables
			if (strlen(g_pszEnv) > 0)
			{
				strcat(pszTemp, "|");
				strcat(pszTemp, g_pszEnv);
			}
				
			// Save the string in the output array
			ppEnv[i] = new char[strlen(pszTemp)+1];
			strcpy(ppEnv[i], pszTemp);
		}
		ppEnv[g_nNproc] = NULL;
	}
	else
	{
		// Figure out how many total processes
		HostNode *n = g_pHosts;
		while (n)
		{
			nProc += n->nSMPProcs;
			n = n->next;
		}
		
		// Allocate an array to hold the environment strings
		ppEnv = new char*[nProc+1];
		
		// Create the strings
		while (g_pHosts)
		{
			nShmLow = iproc;
			nShmHigh = iproc + g_pHosts->nSMPProcs - 1;
			for (int i=0; i<g_pHosts->nSMPProcs; i++)
			{
				// Create the string
				sprintf(pszTemp, "MPICH_NPROC=%d|MPICH_IPROC=%d|MPICH_SHM_LOW=%d|MPICH_SHM_HIGH=%d", 
					nProc, iproc, nShmLow, nShmHigh);
				
				// Add any user variables
				if (strlen(g_pszEnv) > 0)
				{
					strcat(pszTemp, "|");
					strcat(pszTemp, g_pszEnv);
				}
				
				// Save the string in the output array
				ppEnv[iproc] = new char[strlen(pszTemp)+1];
				strcpy(ppEnv[iproc], pszTemp);
				
				// Increment the counter
				iproc++;
			}
			
			// Delete the current node and advance to the next
			HostNode *n = g_pHosts;
			g_pHosts = g_pHosts->next;
			delete n;
		}
		ppEnv[nProc] = NULL;
	}

	return ppEnv;
}
